'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function (_ref) {
    var t = _ref.types;

    return {
        visitor: {
            Identifier: function Identifier(babelPath, state) {
                var _state$opts = state.opts,
                    filename = _state$opts.filename,
                    childFolderName = _state$opts.childFolderName,
                    separator = _state$opts.separator;

                var oFilename = filename || 'omodule.js';
                var oChildFolderName = childFolderName || 'omodules';
                separator = separator || '/';
                var dirname = getFolderName(state.file.opts.filename);

                if (babelPath.node.name === '__omodule_name') {
                    babelPath.replaceWith(t.stringLiteral(getOmoduleName(dirname, state.file.opts.sourceRoot, oFilename)));
                }
                if (babelPath.node.name === '__omodule_namespace') {
                    var _dirname = getFolderName(state.file.opts.filename);
                    babelPath.replaceWith(t.stringLiteral(getOmoduleNameSpace(_dirname, state.file.opts.sourceRoot, oFilename, separator)));
                }
                if (babelPath.node.name === '__omodule_filename') {
                    babelPath.replaceWith(t.stringLiteral(_path2.default.basename(state.file.opts.filename)));
                }
                if (babelPath.node.name === '__omodule_childnames') {
                    var names = getOmoduleChildNames(state.file.opts.filename, state.file.opts.sourceRoot, oFilename, oChildFolderName);

                    babelPath.replaceWith(t.arrayExpression(names.map(function (name) {
                        return t.stringLiteral(name);
                    })));
                }
            }
        }
    };
};

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var isFile = function isFile(pathString) {
    return _fs2.default.lstatSync(pathString).isFile();
};

var isDirectory = function isDirectory(pathString) {
    return _fs2.default.lstatSync(pathString).isDirectory();
};

var getFolderName = function getFolderName(pathString) {
    return _path2.default.join(pathString, '..');
};

var isRoot = function isRoot(dirname, sourceRoot) {
    if (!dirname) {
        return true;
    }
    if (sourceRoot && sourceRoot === dirname) {
        return true;
    }
    var parentDir = getFolderName(dirname);
    if (parentDir === dirname) {
        return true;
    }

    return false;
};

var hasOmoduleFileFun = function hasOmoduleFileFun(oFilename) {
    return function (dirname) {
        var files = _fs2.default.readdirSync(dirname);
        return files.indexOf(oFilename) > -1 && isFile(_path2.default.join(dirname, oFilename));
    };
};

var getOmoduleFolderPath = function getOmoduleFolderPath(dirname, sourceRoot, oFilename) {
    var hasOmoduleFile = hasOmoduleFileFun(oFilename);
    if (hasOmoduleFile(dirname)) {
        return dirname;
    } else {
        var parentDir = getFolderName(dirname);
        if (isRoot(parentDir, sourceRoot)) {
            throw new Error('file: ' + _path2.default.join(dirname, oFilename) + ' not be included in any \'omodule\'.');
        }
        return getOmoduleFolderPath(parentDir, sourceRoot, oFilename);
    }
};

var getOmoduleName = function getOmoduleName(dirname, sourceRoot, oFilename) {
    return _path2.default.basename(getOmoduleFolderPath(dirname, sourceRoot, oFilename));
};

var getParentOmoduleFolderPath = function getParentOmoduleFolderPath(omoduleFolderPath, sourceRoot, oFilename) {
    var hasOmoduleFile = hasOmoduleFileFun(oFilename);
    var omodulesFloderPath = getFolderName(omoduleFolderPath);
    var parentDir = getFolderName(omodulesFloderPath);

    if (isRoot(parentDir, sourceRoot)) {
        return null;
    } else {
        if (hasOmoduleFile(parentDir)) {
            return parentDir;
        } else {
            return null;
        }
    }
};

var getOmoduleNameSpace = function getOmoduleNameSpace(dirname, sourceRoot, oFilename, separator) {
    var subNamespace = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : '';

    var omoduleFolderPath = getOmoduleFolderPath(dirname, sourceRoot, oFilename);
    var omoduleFolderName = _path2.default.basename(omoduleFolderPath);
    var parentOmoduleFolderPath = getParentOmoduleFolderPath(omoduleFolderPath, sourceRoot, oFilename);
    var namespace = subNamespace ? omoduleFolderName + separator + subNamespace : omoduleFolderName;
    if (parentOmoduleFolderPath) {
        return getOmoduleNameSpace(parentOmoduleFolderPath, sourceRoot, oFilename, separator, namespace);
    }

    return namespace;
};

var getOmoduleChildNames = function getOmoduleChildNames(filename, sourceRoot, oFilename, oChildFolderName) {
    var hasOmoduleFile = hasOmoduleFileFun(oFilename);
    var dirname = getFolderName(filename);
    var omoduleFolderPath = getOmoduleFolderPath(dirname, sourceRoot, oFilename);
    var files = _fs2.default.readdirSync(omoduleFolderPath) || [];

    var childOmoduleFolderPath = _path2.default.join(omoduleFolderPath, oChildFolderName);
    if (files.indexOf(oChildFolderName) > -1 && isDirectory(childOmoduleFolderPath)) {
        var childOmoduleFolderFiles = _fs2.default.readdirSync(childOmoduleFolderPath) || [];
        return childOmoduleFolderFiles.reduce(function (names, filename) {
            var filePath = _path2.default.join(childOmoduleFolderPath, filename);
            if (isDirectory(filePath) && hasOmoduleFile(filePath)) {
                return [].concat(_toConsumableArray(names), [filename]);
            } else {
                return names;
            }
        }, []);
    } else {
        return [];
    }
};